@@ -29,7 +29,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from libqtile import drawer, hook, window
+from libqtile import hook
 from libqtile.layout.base import Layout
 
 to_superscript = dict(zip(map(ord, u'0123456789'), map(ord, u'⁰¹²³⁴⁵⁶⁷⁸⁹')))
@@ -263,6 +263,9 @@ class Window(TreeNode):
         if self.window is layout._focused:
             fg = layout.active_fg
             bg = layout.active_bg
+        elif self.window.urgent:
+            fg = layout.urgent_fg
+            bg = layout.urgent_bg
         else:
             fg = layout.inactive_fg
             bg = layout.inactive_bg
@@ -358,6 +361,8 @@ class TreeTab(Layout):
         ("bg_color", "000000", "Background color of tabs"),
         ("active_bg", "000080", "Background color of active tab"),
         ("active_fg", "ffffff", "Foreground color of active tab"),
+        ("urgent_bg", "ff0000", "Background color of urgent tab"),
+        ("urgent_fg", "ffffff", "Foreground color of urgent tab"),
         ("inactive_bg", "606060", "Background color of inactive tab"),
         ("inactive_fg", "ffffff", "Foreground color of inactive tab"),
         ("margin_left", 6, "Left margin of tab panel"),
@@ -379,7 +384,6 @@ class TreeTab(Layout):
         ("section_left", 4, "Left margin of section label"),
         ("panel_width", 150, "Width of the left panel"),
         ("sections", ['Default'], "Foreground color of inactive tab"),
-        ("name", "treetab", "Name of this layout."),
         ("previous_on_rm", False, "Focus previous window on close instead of first."),
     ]
 
@@ -400,9 +404,6 @@ class TreeTab(Layout):
         c._tree = Root(self.sections)
         return c
 
-    def _get_window(self):
-        return self._focused
-
     def focus(self, win):
         self._focused = win
 
@@ -456,23 +457,18 @@ class TreeTab(Layout):
         self.draw_panel()
 
     def _create_panel(self, screen_rect):
-        self._panel = window.Internal.create(
-            self.group.qtile,
+        self._panel = self.group.qtile.core.create_internal(
             screen_rect.x,
             screen_rect.y,
             self.panel_width,
             100
         )
         self._create_drawer(screen_rect)
-        self._panel.handle_Expose = self._handle_Expose
-        self._panel.handle_ButtonPress = self._handle_ButtonPress
-        self.group.qtile.windows_map[self._panel.window.wid] = self._panel
+        self._panel.process_window_expose = self.draw_panel
+        self._panel.process_button_click = self.process_button_click
         hook.subscribe.client_name_updated(self.draw_panel)
         hook.subscribe.focus_change(self.draw_panel)
 
-    def _handle_Expose(self, e):  # noqa: N802
-        self.draw_panel()
-
     def draw_panel(self, *args):
         if not self._panel:
             return
@@ -480,8 +476,8 @@ class TreeTab(Layout):
         self._tree.draw(self, 0)
         self._drawer.draw(offsetx=0, width=self.panel_width)
 
-    def _handle_ButtonPress(self, event):  # noqa: N802
-        node = self._tree.button_press(event.event_x, event.event_y)
+    def process_button_click(self, x, y, _buttom):
+        node = self._tree.button_press(x, y)
         if node:
             self.group.focus(node.window, False)
 
@@ -535,7 +531,6 @@ class TreeTab(Layout):
             return tree
 
         d = Layout.info(self)
-        # Sort client names to work around an internal difference betwen Python 3.5 and 3.6+
         d["clients"] = sorted([x.name for x in self._nodes])
         d["sections"] = [x.title for x in self._tree.children]
 
@@ -716,11 +711,9 @@ class TreeTab(Layout):
 
     def _create_drawer(self, screen_rect):
         if self._drawer is None:
-            self._drawer = drawer.Drawer(
-                self.group.qtile,
-                self._panel.window.wid,
+            self._drawer = self._panel.create_drawer(
                 self.panel_width,
-                screen_rect.height
+                screen_rect.height,
             )
         self._drawer.clear(self.bg_color)
         self._layout = self._drawer.textlayout(
