@@ -19,29 +19,27 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import functools
+import asyncio
 import glob
 import importlib
 import os
 import traceback
-import warnings
 from collections import defaultdict
 from collections.abc import Sequence
 from random import randint
 from shutil import which
+from typing import List, Tuple, Union
+
+try:
+    from dbus_next import Message, Variant  # type: ignore
+    from dbus_next.aio import MessageBus  # type: ignore
+    from dbus_next.constants import BusType, MessageType  # type: ignore
+    has_dbus = True
+except ImportError:
+    has_dbus = False
 
 from libqtile.log_utils import logger
 
-_can_notify = False
-try:
-    import gi
-    gi.require_version("Notify", "0.7")  # type: ignore
-    from gi.repository import Notify  # type: ignore
-    Notify.init("Qtile")
-    _can_notify = True
-except ImportError as e:
-    logger.warning("Failed to import dependencies for notifications: %s" % e)
-
 
 class QtileError(Exception):
     pass
@@ -68,6 +66,9 @@ def shuffle_down(lst):
         lst.append(c)
 
 
+ColorType = Union[str, Tuple[int, int, int], Tuple[int, int, int, float]]
+
+
 def rgb(x):
     """
         Returns a valid RGBA tuple.
@@ -109,6 +110,43 @@ def hex(x):
     return '#%02x%02x%02x' % (int(r * 255), int(g * 255), int(b * 255))
 
 
+def has_transparency(colour: Union[ColorType, List[ColorType]]):
+    """
+    Returns True if the colour is not fully opaque.
+
+    Where a list of colours is passed, returns True if any
+    colour is not fully opaque.
+    """
+    def has_alpha(col):
+        return rgb(col)[3] < 1
+
+    if isinstance(colour, (str, tuple)):
+        return has_alpha(colour)
+
+    elif isinstance(colour, list):
+        print([c for c in colour])
+        return any([has_transparency(c) for c in colour])
+
+    return False
+
+
+def remove_transparency(colour: Union[ColorType, List[ColorType]]):
+    """
+    Returns a tuple of (r, g, b) with no alpha.
+    """
+    def remove_alpha(col):
+        stripped = tuple(x * 255.0 for x in rgb(col)[:3])
+        return stripped
+
+    if isinstance(colour, (str, tuple)):
+        return remove_alpha(colour)
+
+    elif isinstance(colour, list):
+        return [remove_transparency(c) for c in colour]
+
+    return (0, 0, 0)
+
+
 def scrub_to_utf8(text):
     if not text:
         return ""
@@ -118,49 +156,6 @@ def scrub_to_utf8(text):
         return text.decode("utf-8", "ignore")
 
 
-# WARNINGS
-class UnixCommandNotFound(Warning):
-    pass
-
-
-class UnixCommandRuntimeError(Warning):
-    pass
-
-
-def catch_exception_and_warn(warning=Warning, return_on_exception=None,
-                             excepts=Exception):
-    """
-    .. function:: warn_on_exception(func, [warning_class, return_on_failure,
-            excepts])
-        attempts to call func. catches exception or exception tuple and issues
-        a warning instead. returns value of return_on_failure when the
-        specified exception is raised.
-
-        :param func: a callable to be wrapped
-        :param warning: the warning class to issue if an exception is
-            raised
-        :param return_on_exception: the default return value of the function
-            if an exception is raised
-        :param excepts: an exception class (or tuple of exception classes) to
-            catch during the execution of func
-        :type excepts: Exception or tuple of Exception classes
-        :type warning: Warning
-        :rtype: a callable
-    """
-    def decorator(func):
-        @functools.wraps(func)
-        def wrapper(*args, **kwargs):
-            return_value = return_on_exception
-            try:
-                return_value = func(*args, **kwargs)
-            except excepts as err:
-                logger.warning(str(err))
-                warnings.warn(str(err), warning)
-            return return_value
-        return wrapper
-    return decorator
-
-
 def get_cache_dir():
     """
     Returns the cache directory and create if it doesn't exists
@@ -239,16 +234,51 @@ def send_notification(title, message, urgent=False, timeout=10000, id=None):
     passed when calling this function again to replace that notification. See:
     https://developer.gnome.org/notification-spec/
     """
-    if _can_notify and Notify.get_server_info()[0]:
-        notifier = Notify.Notification.new(title, message)
-        if urgent:
-            notifier.set_urgency(Notify.Urgency.CRITICAL)
-        notifier.set_timeout(timeout)
-        if id is None:
-            id = randint(10, 1000)
-        notifier.set_property('id', id)
-        notifier.show()
-        return id
+    if not has_dbus:
+        logger.warning(
+            "dbus-next is not installed. Unable to send notifications."
+        )
+        return -1
+
+    id = randint(10, 1000) if id is None else id
+    urgency = 2 if urgent else 1
+
+    try:
+        loop = asyncio.get_running_loop()
+    except RuntimeError:
+        logger.warning("Eventloop has not started. Cannot send notification.")
+    else:
+        loop.create_task(_notify(title, message, urgency, timeout, id))
+
+    return id
+
+
+async def _notify(title, message, urgency, timeout, id):
+    notification = ["qtile",  # Application name
+                    id,  # id
+                    "",  # icon
+                    title,  # summary
+                    message,  # body
+                    [],  # actions
+                    {"urgency": Variant("y", urgency)},  # hints
+                    timeout]  # timeout
+
+    bus, msg = await _send_dbus_message(True,
+                                        MessageType.METHOD_CALL,
+                                        "org.freedesktop.Notifications",
+                                        "org.freedesktop.Notifications",
+                                        "/org/freedesktop/Notifications",
+                                        "Notify",
+                                        "susssasa{sv}i",
+                                        notification)
+
+    if msg.message_type == MessageType.ERROR:
+        logger.warning("Unable to send notification. "
+                       "Is a notification server running?")
+
+    # a new bus connection is made each time a notification is sent so
+    # we disconnect when the notification is done
+    bus.disconnect()
 
 
 def guess_terminal(preference=None):
@@ -310,3 +340,76 @@ def scan_files(dirpath, *names):
         files[name].extend(found)
 
     return files
+
+
+async def _send_dbus_message(session_bus, message_type, destination, interface,
+                             path, member, signature, body):
+    """
+    Private method to send messages to dbus via dbus_next.
+
+    Returns a tuple of the bus object and message response.
+    """
+    if session_bus:
+        bus_type = BusType.SESSION
+    else:
+        bus_type = BusType.SYSTEM
+
+    if isinstance(body, str):
+        body = [body]
+
+    bus = await MessageBus(bus_type=bus_type).connect()
+
+    msg = await bus.call(
+        Message(message_type=message_type,
+                destination=destination,
+                interface=interface,
+                path=path,
+                member=member,
+                signature=signature,
+                body=body))
+
+    return bus, msg
+
+
+async def add_signal_receiver(callback, session_bus=False, signal_name=None,
+                              dbus_interface=None, bus_name=None, path=None):
+    """
+    Helper function which aims to recreate python-dbus's add_signal_receiver
+    method in dbus_next with asyncio calls.
+
+    Returns True if subscription is successful.
+    """
+    if not has_dbus:
+        logger.warning(
+            "dbus-next is not installed. "
+            "Unable to subscribe to signals"
+        )
+        return False
+
+    match_args = {
+        "type": "signal",
+        "sender": bus_name,
+        "member": signal_name,
+        "path": path,
+        "interface": dbus_interface
+    }
+
+    rule = ",".join("{}='{}'".format(k, v)
+                    for k, v in match_args.items() if v)
+
+    bus, msg = await _send_dbus_message(session_bus,
+                                        MessageType.METHOD_CALL,
+                                        "org.freedesktop.DBus",
+                                        "org.freedesktop.DBus",
+                                        "/org/freedesktop/DBus",
+                                        "AddMatch",
+                                        "s",
+                                        rule)
+
+    # Check if message sent successfully
+    if msg.message_type == MessageType.METHOD_RETURN:
+        bus.add_message_handler(callback)
+        return True
+
+    else:
+        return False
